@@ -3,6 +3,7 @@
 #include "event_loop.h"
 #include "event.h"
 #include "volume.h"
+#include "power.h"
 
 struct bar_item* bar_item_create() {
   struct bar_item* bar_item = malloc(sizeof(struct bar_item));
@@ -55,6 +56,7 @@ void bar_item_init(struct bar_item* bar_item, struct bar_item* default_item) {
   popup_init(&bar_item->popup, bar_item);
   graph_init(&bar_item->graph);
   alias_init(&bar_item->alias);
+  slider_init(&bar_item->slider);
   
   if (default_item) bar_item_inherit_from_item(bar_item, default_item);
 }
@@ -173,7 +175,15 @@ void bar_item_needs_update(struct bar_item* bar_item) {
   bar_item->needs_update = true;
 }
 
-void bar_item_on_click(struct bar_item* bar_item, uint32_t type, uint32_t modifier) {
+void bar_item_on_drag(struct bar_item* bar_item, CGPoint point) {
+  if (bar_item->has_slider) {
+    if (slider_handle_drag(&bar_item->slider, point)) {
+      bar_item_needs_update(bar_item);
+    }
+  }
+}
+
+void bar_item_on_click(struct bar_item* bar_item, uint32_t type, uint32_t modifier, CGPoint point) {
   if (!bar_item) return;
 
   env_vars_set(&bar_item->signal_args.env_vars,
@@ -184,6 +194,24 @@ void bar_item_on_click(struct bar_item* bar_item, uint32_t type, uint32_t modifi
                string_copy("MODIFIER"),
                string_copy(get_modifier_description(modifier)));
 
+  if (bar_item->has_slider) {
+    if (bar_item->slider.is_dragged
+        || CGRectContainsPoint(bar_item->slider.background.bounds, point)) {
+      if (slider_handle_drag(&bar_item->slider, point)) {
+        bar_item_needs_update(bar_item);
+      }
+
+      char perc_str[8];
+      snprintf(perc_str, 8, "%d", bar_item->slider.percentage);
+      env_vars_set(&bar_item->signal_args.env_vars,
+                   string_copy("PERCENTAGE"),
+                   string_copy(perc_str)           );
+      bar_item->slider.is_dragged = false;
+    } else {
+      return;
+    }
+  }
+
   if (bar_item->click_script && strlen(bar_item->click_script) > 0)
     fork_exec(bar_item->click_script, &bar_item->signal_args.env_vars);
   if (bar_item->update_mask & UPDATE_MOUSE_CLICKED)
@@ -274,25 +302,43 @@ static void bar_item_set_event_port(struct bar_item* bar_item, char* bs_name) {
   bar_item->event_port = port;
 }
 
-void bar_item_set_name(struct bar_item* bar_item, char* name) {
-  if (!name) return;
+bool bar_item_set_name(struct bar_item* bar_item, char* name) {
+  if (!name) return false;
 
   if (bar_item->name && strcmp(bar_item->name, name) == 0) {
     free(name);
-    return;
+    return true;
   }
 
+  if (strlen(name) == 0) return false;
+
   if (name != bar_item->name && bar_item->name) free(bar_item->name);
   bar_item->name = name;
   env_vars_set(&bar_item->signal_args.env_vars,
                string_copy("NAME"),
                string_copy(name)               );
+  return true;
 }
 
-void bar_item_set_type(struct bar_item* bar_item, char type) {
-  bar_item->type = type;
+bool bar_item_set_type(struct bar_item* bar_item, char* type) {
+  bool success = true;
 
-  if (type == BAR_COMPONENT_SPACE) {
+  if (string_equals(type, TYPE_SPACE)) {
+    bar_item->type = BAR_COMPONENT_SPACE;
+  } else if (string_equals(type, TYPE_ALIAS)) {
+    bar_item->type = BAR_COMPONENT_ALIAS;
+  } else if (string_equals(type, TYPE_GROUP)) {
+    bar_item->type = BAR_COMPONENT_GROUP;
+  } else if (string_equals(type, TYPE_GRAPH)) {
+    bar_item->type = BAR_COMPONENT_GRAPH;
+  } else if (string_equals(type, TYPE_SLIDER)) {
+    bar_item->type = BAR_COMPONENT_SLIDER;
+  } else {
+    bar_item->type = BAR_ITEM;
+    success = string_equals(type, TYPE_ITEM);
+  }
+
+  if (bar_item->type == BAR_COMPONENT_SPACE) {
     if (!bar_item->script) { 
         bar_item_set_script(bar_item,
             string_copy("sketchybar -m --set $NAME icon.highlight=$SELECTED"));
@@ -312,32 +358,57 @@ void bar_item_set_type(struct bar_item* bar_item, char type) {
                  string_copy("DID"),
                  string_copy("0")                );
   }
-  else if (type == BAR_COMPONENT_ALIAS) {
+  else if (bar_item->type == BAR_COMPONENT_ALIAS) {
     bar_item->update_frequency = 1;
     bar_item->has_alias = true;
     bar_item->updates_only_when_shown = true;
   }
-  else if (type == BAR_COMPONENT_GRAPH) {
+  else if (bar_item->type == BAR_COMPONENT_GRAPH) {
     bar_item->has_graph = true;
   }
-  else if (type == BAR_COMPONENT_GROUP) {
+  else if (bar_item->type == BAR_COMPONENT_SLIDER) {
+    bar_item->has_slider = true;
+  }
+  else if (bar_item->type == BAR_COMPONENT_GROUP) {
     bar_item->group = group_create();
     group_init(bar_item->group);
     group_add_member(bar_item->group, bar_item);
   }
+  return success;
 }
 
-void bar_item_set_position(struct bar_item* bar_item, char position) {
-  bar_item->position = position;
-  if (position != POSITION_POPUP)
-    bar_item->align = position;
+bool bar_item_set_position(struct bar_item* bar_item, char* position) {
+  if (!position || strlen(position) == 0) return false;
+
+  switch (position[0]) {
+    case POSITION_LEFT:
+      break;
+    case POSITION_CENTER_LEFT:
+      break;
+    case POSITION_CENTER:
+      break;
+    case POSITION_CENTER_RIGHT:
+      break;
+    case POSITION_RIGHT:
+      break;
+    case POSITION_POPUP:
+      break;
+    default:
+      return false;
+  }
+
+  bar_item->position = position[0];
+  if (position[0] != POSITION_POPUP)
+    bar_item->align = position[0];
+  return true;
 }
 
 static uint32_t bar_item_get_content_length(struct bar_item* bar_item) {
   int length = text_get_length(&bar_item->icon, false)
          + text_get_length(&bar_item->label, false)
-         + (bar_item->has_graph ? graph_get_length(&bar_item->graph) : 0)
-         + (bar_item->has_alias ? alias_get_length(&bar_item->alias) : 0);
+         + (bar_item->has_graph  ? graph_get_length(&bar_item->graph) : 0)
+         + (bar_item->has_slider ? slider_get_length(&bar_item->slider) : 0)
+         + (bar_item->has_alias  ? alias_get_length(&bar_item->alias) : 0);
  
   return max(length, 0);
 }
@@ -474,8 +545,11 @@ uint32_t bar_item_calculate_bounds(struct bar_item* bar_item, uint32_t bar_heigh
     label_position += graph_get_length(&bar_item->graph);
   } else if (bar_item->has_alias) {
     label_position += alias_get_length(&bar_item->alias); 
+  } else if (bar_item->has_slider) {
+    label_position += slider_get_length(&bar_item->slider); 
   }
 
+
   text_calculate_bounds(&bar_item->icon,
                         icon_position,
                         content_y + bar_item->y_offset);
@@ -489,15 +563,22 @@ uint32_t bar_item_calculate_bounds(struct bar_item* bar_item, uint32_t bar_heigh
                            sandwich_position,
                            content_y + bar_item->y_offset);
 
-  if (bar_item->has_graph) {
-    bar_item->graph.bounds.size.height = bar_item->background.enabled
-                                         ? (bar_item->background.bounds.size.height
-                                            - bar_item->background.border_width - 1)
-                                         : (bar_height
-                                            - (g_bar_manager.background.border_width + 1));
+  if (bar_item->has_slider)
+    slider_calculate_bounds(&bar_item->slider,
+                            sandwich_position,
+                            content_y + bar_item->y_offset);
 
-    graph_calculate_bounds(&bar_item->graph, sandwich_position,
-                                             content_y + bar_item->y_offset);
+  if (bar_item->has_graph) {
+    uint32_t height = bar_item->background.enabled
+                      ? (bar_item->background.bounds.size.height
+                         - bar_item->background.border_width - 1)
+                      : (bar_height
+                         - (g_bar_manager.background.border_width + 1));
+
+    graph_calculate_bounds(&bar_item->graph,
+                           sandwich_position,
+                           content_y + bar_item->y_offset,
+                           height                         );
   }
 
   if (bar_item->background.enabled) {
@@ -520,8 +601,8 @@ bool bar_item_clip_needs_update_for_bar(struct bar_item* bar_item, struct bar* b
   bool needs_update = false;
 
   needs_update |= background_clip_needs_update(&bar_item->background, bar)
-               | background_clip_needs_update(&bar_item->icon.background, bar)
-               | background_clip_needs_update(&bar_item->label.background, bar);
+               || background_clip_needs_update(&bar_item->icon.background, bar)
+               || background_clip_needs_update(&bar_item->label.background, bar);
 
   return needs_update;
 }
@@ -549,6 +630,8 @@ void bar_item_draw(struct bar_item* bar_item, CGContextRef context) {
     alias_draw(&bar_item->alias, context);
   if (bar_item->has_graph)
     graph_draw(&bar_item->graph, context);
+  if (bar_item->has_slider)
+    slider_draw(&bar_item->slider, context);
 }
 
 void bar_item_change_space(struct bar_item* bar_item, uint64_t dsid, uint32_t adid) {
@@ -571,11 +654,13 @@ static void bar_item_clear_pointers(struct bar_item* bar_item) {
   text_clear_pointers(&bar_item->icon);
   text_clear_pointers(&bar_item->label);
   background_clear_pointers(&bar_item->background);
+  slider_clear_pointers(&bar_item->slider);
 }
 
 void bar_item_inherit_from_item(struct bar_item* bar_item, struct bar_item* ancestor) {
   text_destroy(&bar_item->icon);
   text_destroy(&bar_item->label);
+  text_destroy(&bar_item->slider.knob);
   
   char* name = bar_item->name;
   char* script = bar_item->script;
@@ -593,6 +678,12 @@ void bar_item_inherit_from_item(struct bar_item* bar_item, struct bar_item* ance
   text_set_string(&bar_item->icon, string_copy(ancestor->icon.string), true);
   text_set_string(&bar_item->label, string_copy(ancestor->label.string), true);
 
+  text_set_font(&bar_item->slider.knob,
+                string_copy(ancestor->slider.knob.font_name), true);
+
+  text_set_string(&bar_item->slider.knob,
+                  string_copy(ancestor->slider.knob.string), true);
+
   if (ancestor->script)
     bar_item_set_script(bar_item, string_copy(ancestor->script));
   if (ancestor->click_script)
@@ -631,13 +722,9 @@ void bar_item_destroy(struct bar_item* bar_item) {
   text_destroy(&bar_item->icon);
   text_destroy(&bar_item->label);
 
-  if (bar_item->has_graph) {
-    graph_destroy(&bar_item->graph);
-  }
-
-  if (bar_item->has_alias) {
-    alias_destroy(&bar_item->alias);
-  }
+  graph_destroy(&bar_item->graph);
+  alias_destroy(&bar_item->alias);
+  slider_destroy(&bar_item->slider);
 
   if (bar_item->group && bar_item->type == BAR_COMPONENT_GROUP)
     group_destroy(bar_item->group);
@@ -668,6 +755,9 @@ void bar_item_serialize(struct bar_item* bar_item, FILE* rsp) {
     case BAR_COMPONENT_GROUP:
       snprintf(type, 32, "bracket");
       break;
+    case BAR_COMPONENT_SLIDER:
+      snprintf(type, 32, "slider");
+      break;
     case BAR_COMPONENT_GRAPH:
       snprintf(type, 32, "graph");
       break;
@@ -715,6 +805,8 @@ void bar_item_serialize(struct bar_item* bar_item, FILE* rsp) {
                "\t\t\"ignore_association\": \"%s\",\n"
                "\t\t\"y_offset\": %d,\n"
                "\t\t\"width\": %d,\n"
+               "\t\t\"padding_left\": %d,\n"
+               "\t\t\"padding_right\": %d,\n"
                "\t\t\"background\": {\n",
                bar_item->name,
                type,
@@ -724,6 +816,8 @@ void bar_item_serialize(struct bar_item* bar_item, FILE* rsp) {
                bar_item->associated_display,
                format_bool(bar_item->ignore_association),
                bar_item->y_offset,
+               bar_item->background.padding_left,
+               bar_item->background.padding_right,
                bar_item->has_const_width ? bar_item->custom_width : -1);
 
   background_serialize(&bar_item->background, "\t\t\t", rsp, true);
@@ -787,6 +881,10 @@ void bar_item_serialize(struct bar_item* bar_item, FILE* rsp) {
     fprintf(rsp, ",\n\t\"graph\": {\n");
     graph_serialize(&bar_item->graph, "\t\t", rsp);
     fprintf(rsp, "\n\t}");
+  } else if (bar_item->type == BAR_COMPONENT_SLIDER) {
+    fprintf(rsp, ",\n\t\"slider\": {\n");
+    slider_serialize(&bar_item->slider, "\t\t", rsp);
+    fprintf(rsp, "\n\t}");
   }
 
   fprintf(rsp, "\n}\n");
@@ -800,44 +898,63 @@ void bar_item_parse_set_message(struct bar_item* bar_item, char* message, FILE*
   if (key_value_pair.key && key_value_pair.value) {
     struct token subdom = { key_value_pair.key, strlen(key_value_pair.key) };
     struct token entry = { key_value_pair.value, strlen(key_value_pair.value)};
-    if (token_equals(subdom, SUB_DOMAIN_ICON))
+    if (token_equals(subdom, SUB_DOMAIN_ICON)) {
       needs_refresh = text_parse_sub_domain(&bar_item->icon,
                                             rsp,
                                             entry,
                                             message         );
-
-    else if (token_equals(subdom, SUB_DOMAIN_LABEL))
+    }
+    else if (token_equals(subdom, SUB_DOMAIN_LABEL)) {
       needs_refresh = text_parse_sub_domain(&bar_item->label,
                                             rsp,
                                             entry,
                                             message          );
-
-    else if (token_equals(subdom, SUB_DOMAIN_BACKGROUND))
+    }
+    else if (token_equals(subdom, SUB_DOMAIN_BACKGROUND)) {
       needs_refresh = background_parse_sub_domain(&bar_item->background,
                                                   rsp,
                                                   entry,
                                                   message               );
-
-    else if (token_equals(subdom, SUB_DOMAIN_GRAPH))
-      needs_refresh = graph_parse_sub_domain(&bar_item->graph,
-                                             rsp,
-                                             entry,
-                                             message          );
-
-    else if (token_equals(subdom, SUB_DOMAIN_POPUP))
+    }
+    else if (token_equals(subdom, SUB_DOMAIN_POPUP)) {
       needs_refresh = popup_parse_sub_domain(&bar_item->popup,
                                              rsp,
                                              entry,
                                              message          );
-
-    else if (token_equals(subdom, SUB_DOMAIN_ALIAS))
-      needs_refresh = alias_parse_sub_domain(&bar_item->alias,
-                                             rsp,
-                                             entry,
-                                             message          );
-
-    else
+    }
+    else if (token_equals(subdom, SUB_DOMAIN_GRAPH)) {
+      if (bar_item->has_graph || bar_item == &g_bar_manager.default_item) {
+        needs_refresh = graph_parse_sub_domain(&bar_item->graph,
+                                               rsp,
+                                               entry,
+                                               message          );
+      } else {
+        respond(rsp, "[!] Item (%s): Trying to set a graph property on a non-graph item\n", bar_item->name);
+      }
+    }
+    else if (token_equals(subdom, SUB_DOMAIN_ALIAS)) {
+      if (bar_item->has_alias || bar_item == &g_bar_manager.default_item) {
+        needs_refresh = alias_parse_sub_domain(&bar_item->alias,
+                                               rsp,
+                                               entry,
+                                               message          );
+      } else {
+        respond(rsp, "[!] Item (%s): Trying to set an alias property on a non-alias item\n", bar_item->name);
+      }
+    }
+    else if (token_equals(subdom, SUB_DOMAIN_SLIDER)) {
+      if (bar_item->has_slider || bar_item == &g_bar_manager.default_item) {
+        needs_refresh = slider_parse_sub_domain(&bar_item->slider,
+                                                rsp,
+                                                entry,
+                                                message           );
+      } else {
+        respond(rsp, "[!] Item (%s): Trying to set a slider property on a non-slider item\n", bar_item->name);
+      }
+    }
+    else {
       respond(rsp, "[!] Item (%s): Invalid subdomain '%s'\n", bar_item->name, subdom.text);
+    }
   }
   else if (token_equals(property, PROPERTY_ICON)) {
     needs_refresh = text_set_string(&bar_item->icon,
@@ -901,7 +1018,7 @@ void bar_item_parse_set_message(struct bar_item* bar_item, char* message, FILE*
     bar_item->update_frequency = token_to_uint32t(get_token(&message));
   } else if (token_equals(property, PROPERTY_POSITION)) {
     struct token position = get_token(&message);
-    bar_item_set_position(bar_item, position.text[0]);
+    bar_item_set_position(bar_item, position.text);
     struct key_value_pair key_value_pair = get_key_value_pair(position.text,
                                                               '.'           );
 
@@ -971,6 +1088,20 @@ void bar_item_parse_set_message(struct bar_item* bar_item, char* message, FILE*
             bar_item->y_offset,
             token_to_int(token)  );
 
+  } else if (token_equals(property, PROPERTY_PADDING_LEFT)) {
+    struct token token = get_token(&message);
+    ANIMATE(background_set_padding_left,
+            &bar_item->background,
+            bar_item->background.padding_left,
+            token_to_int(token)               );
+
+  } else if (token_equals(property, PROPERTY_PADDING_RIGHT)) {
+    struct token token = get_token(&message);
+    ANIMATE(background_set_padding_right,
+            &bar_item->background,
+            bar_item->background.padding_right,
+            token_to_int(token)                );
+
   } else if (token_equals(property, PROPERTY_BLUR_RADIUS)) {
     struct token token = get_token(&message);
     ANIMATE(bar_item_set_blur_radius,
