@@ -451,11 +451,9 @@ void bar_manager_update_space_components(struct bar_manager* bar_manager, bool f
       uint32_t space_did = display_id_for_space(space);
       if (space_did) {
         bar_item->associated_display = 1 << (display_arrangement(space_did));
-        bar_item->drawing = true;
       }
       else {
         bar_item->associated_display = 1 << 30;
-        bar_item->drawing = false;
       }
     }
     for (int j = 0; j < bar_manager->bar_count; j++) {
@@ -711,9 +709,6 @@ void bar_manager_handle_mouse_exited_global(struct bar_manager* bar_manager) {
 
 void bar_manager_handle_mouse_entered(struct bar_manager* bar_manager, struct bar_item* bar_item) {
   if (!bar_item || bar_item->mouse_over) return;
-  for (int i = 0; i < bar_manager->bar_item_count; i++)
-    bar_item_mouse_exited(bar_item);
-
   bar_item_mouse_entered(bar_item);
 }
 
@@ -750,6 +745,16 @@ void bar_manager_handle_brightness_change(struct bar_manager* bar_manager, float
   env_vars_destroy(&env_vars);
 }
 
+void bar_manager_handle_power_source_change(struct bar_manager* bar_manager, char* state) {
+  struct env_vars env_vars;
+  env_vars_init(&env_vars);
+  env_vars_set(&env_vars, string_copy("INFO"), string_copy(state));
+  bar_manager_custom_events_trigger(bar_manager,
+                                    COMMAND_SUBSCRIBE_POWER_SOURCE_CHANGE,
+                                    &env_vars                             );
+  env_vars_destroy(&env_vars);
+}
+
 void bar_manager_handle_front_app_switch(struct bar_manager* bar_manager, char* info) {
   struct env_vars env_vars;
   env_vars_init(&env_vars);
