@@ -3,7 +3,6 @@
 
 extern struct event_loop g_event_loop;
 extern struct bar_manager g_bar_manager;
-extern bool g_verbose;
 
 static struct bar_item** get_bar_items_for_regex(struct token reg, FILE* rsp, uint32_t* count) {
   struct bar_item** bar_items = NULL;
@@ -178,15 +177,35 @@ static void handle_domain_add(FILE* rsp, struct token domain, char* message) {
   } 
   struct bar_item* bar_item = bar_manager_create_item(&g_bar_manager);
 
-  bar_item_set_type(bar_item, command.text[0]);
-  bar_item_set_position(bar_item, position.text[0]);
-  bar_item_set_name(bar_item, token_to_string(name));
+  if (!bar_item_set_type(bar_item, command.text)) {
+    respond(rsp, "[?] Add %s: Invalid type '%s', assuming 'item'\n",
+                 name.text,
+                 command.text                                           );
+  }
+
+  if (!bar_item_set_position(bar_item, position.text)
+      && bar_item->type != BAR_COMPONENT_GROUP) {
+    respond(rsp, "[!] Add %s: Illegal position '%s'\n", name.text,
+                                                        position.text);
+    bar_manager_remove_item(&g_bar_manager, bar_item);
+    return;
+  }
+
+  if (!bar_item_set_name(bar_item, token_to_string(name))) {
+    respond(rsp, "[!] Add: Illegal name '%s'\n", name.text);
+    bar_manager_remove_item(&g_bar_manager, bar_item);
+    return;
+  }
 
   if (token_equals(command, COMMAND_ADD_ITEM)) {
   } else if (command.length > 0) {
     if (bar_item->type == BAR_COMPONENT_GRAPH) {
       struct token width = get_token(&message);
       graph_setup(&bar_item->graph, token_to_uint32t(width));
+    } 
+    else if (bar_item->type == BAR_COMPONENT_SLIDER) {
+      struct token width = get_token(&message);
+      slider_setup(&bar_item->slider, token_to_uint32t(width));
     }
     else if (bar_item->type == BAR_COMPONENT_ALIAS) {
       char* tmp_name = string_copy(name.text);
@@ -200,6 +219,18 @@ static void handle_domain_add(FILE* rsp, struct token domain, char* message) {
 
       free(tmp_name);
     }
+    else if (bar_item->type == BAR_COMPONENT_SLIDER) {
+      char* tmp_name = string_copy(name.text);
+      struct key_value_pair key_value_pair = get_key_value_pair(tmp_name, ',');
+      if (!key_value_pair.key || !key_value_pair.value)
+        alias_setup(&bar_item->alias, token_to_string(name), NULL);
+      else
+        alias_setup(&bar_item->alias,
+                    string_copy(key_value_pair.key),
+                    string_copy(key_value_pair.value));
+
+      free(tmp_name);
+    }
     else if (bar_item->type == BAR_COMPONENT_GROUP) {
       struct token member = position;
       bool first = true;
@@ -245,9 +276,6 @@ static void handle_domain_add(FILE* rsp, struct token domain, char* message) {
         member = get_token(&message);
       }
     }
-  } else {
-    respond(rsp, "[!] Add %s: Invalid item type '%s'\n", bar_item->name, command.text);
-    return;
   }
 
   if (position.text[0] == POSITION_POPUP) {
@@ -260,11 +288,10 @@ static void handle_domain_add(FILE* rsp, struct token domain, char* message) {
         respond(rsp,
                 "[!] Add (Popup) %s: Item '%s' is not a valid popup host\n",
                 bar_item->name,
-                key_value_pair.value,
-                bar_item->name                                                          );
+                key_value_pair.value);
 
         free(pair);
-        bar_item_set_position(bar_item, POSITION_LEFT);
+        bar_manager_remove_item(&g_bar_manager, bar_item);
         return;
       }
       struct bar_item* target_item = g_bar_manager.bar_items[item_index_for_name];
