@@ -22,11 +22,11 @@ static CFTypeRef window_create_region(struct window *window, CGRect frame) {
 }
 
 void window_create(struct window* window, CGRect frame) {
-  uint64_t set_tags = 0;
+  uint64_t set_tags = kCGSExposeFadeTagBit;
   uint64_t clear_tags = 0;
 
   if (g_bar_manager.sticky) {
-    set_tags = kCGSStickyTagBit | kCGSHighQualityResamplingTagBit;
+    set_tags |= kCGSStickyTagBit;
     clear_tags = kCGSSuperStickyTagBit;
   }
 
@@ -42,7 +42,6 @@ void window_create(struct window* window, CGRect frame) {
                                                   &id              );
 
   window->id = (uint32_t)id;
-
   CFRelease(frame_region);
 
   SLSSetWindowResolution(g_connection, window->id, 2.0f);
@@ -69,9 +68,13 @@ void window_create(struct window* window, CGRect frame) {
 
 void windows_freeze() {
   SLSDisableUpdate(g_connection);
+  g_transaction = SLSTransactionCreate(g_connection);
 }
 
 void windows_unfreeze() {
+  SLSTransactionCommit(g_transaction, 0);
+  CFRelease(g_transaction);
+  g_transaction = NULL;
   SLSReenableUpdate(g_connection);
 }
 
@@ -93,10 +96,13 @@ void window_set_frame(struct window* window, CGRect frame) {
 
 void window_move(struct window* window, CGPoint point) {
   window->origin = point;
-  SLSMoveWindow(g_connection, window->id, &point);
 
   if (__builtin_available(macOS 12.0, *)) {
+    // Monterey and later
+    SLSTransactionMoveWindowWithGroup(g_transaction, window->id, point);
   } else {
+    // Big Sur and Previous
+    SLSMoveWindow(g_connection, window->id, &point);
     CFNumberRef number = CFNumberCreate(NULL,
                                         kCFNumberSInt32Type,
                                         &window->id         );
@@ -114,23 +120,27 @@ bool window_apply_frame(struct window* window) {
     CFTypeRef frame_region = window_create_region(window, window->frame);
 
     if (__builtin_available(macOS 13.0, *)) {
-    } else if (window->parent) {
-      window_order(window, window->parent, W_OUT);
-    }
+      SLSSetWindowShape(g_connection, window->id,
+                                      g_nirvana.x,
+                                      g_nirvana.y,
+                                      frame_region);
+    } else {
+      if (window->parent) {
+        SLSOrderWindow(g_connection, window->id, 0, window->parent->id);
+      }
 
-    SLSSetWindowShape(g_connection, window->id, 0, 0, frame_region);
-    CFRelease(frame_region);
-    window_move(window, window->origin);
+      SLSSetWindowShape(g_connection, window->id, 0, 0, frame_region);
 
-    if (window->parent) {
-      CGContextClearRect(window->context, window->frame);
-      CGContextFlush(window->context);
-      if (__builtin_available(macOS 13.0, *)) {
-      } else {
+      if (window->parent) {
+        CGContextClearRect(window->context, window->frame);
+        CGContextFlush(window->context);
         window_order(window, window->parent, window->order_mode);
       }
     }
 
+    CFRelease(frame_region);
+    window_move(window, window->origin);
+
     window->needs_move = false;
     window->needs_resize = false;
     return true;
@@ -143,7 +153,11 @@ bool window_apply_frame(struct window* window) {
 }
 
 void window_send_to_space(struct window* window, uint64_t dsid) {
-  CFArrayRef window_list = cfarray_of_cfnumbers(&window->id, sizeof(uint32_t), 1, kCFNumberSInt32Type);
+  CFArrayRef window_list = cfarray_of_cfnumbers(&window->id,
+                                                sizeof(uint32_t),
+                                                1,
+                                                kCFNumberSInt32Type);
+
   SLSMoveWindowsToManagedSpace(g_connection, window_list, dsid);
   if (CGPointEqualToPoint(window->origin, g_nirvana)) {
     SLSMoveWindow(g_connection, window->id, &g_nirvana);
@@ -151,6 +165,7 @@ void window_send_to_space(struct window* window, uint64_t dsid) {
 }
 
 void window_close(struct window* window) {
+  SLSOrderWindow(g_connection, window->id, 0, 0);
   CGContextRelease(window->context);
   SLSReleaseWindow(g_connection, window->id);
 
@@ -163,22 +178,17 @@ void window_close(struct window* window) {
 }
 
 void window_set_level(struct window* window, uint32_t level) {
-  SLSSetWindowLevel(g_connection, window->id, level);
+  SLSTransactionSetWindowLevel(g_transaction, window->id, level);
 }
 
 void window_order(struct window* window, struct window* parent, int mode) {
-  SLSRemoveFromOrderingGroup(g_connection, window->id);
   if (parent) {
     window->parent = parent;
     if (mode != W_OUT) window->order_mode = mode;
-    SLSOrderWindow(g_connection, window->id, mode, parent->id);
-    SLSAddWindowToWindowOrderingGroup(g_connection,
-                                      parent->id,
-                                      window->id,
-                                      mode         );
+    SLSTransactionOrderWindow(g_transaction, window->id, mode, parent->id);
   } else {
     window->parent = NULL;
-    SLSOrderWindow(g_connection, window->id, mode, 0);
+    SLSTransactionOrderWindow(g_transaction, window->id, mode, 0);
   }
 }
 
